use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.18

use Test::More 0.88;
use Test::EOL;

my @files = (
    'Changes',
    'GPLv3',
    'VERSION',
    'lib/Test/Dist/Zilla.pm',
    'lib/Test/Dist/Zilla/Build.pm',
    'lib/Test/Dist/Zilla/BuiltFiles.pm',
    'lib/Test/Dist/Zilla/Release.pm',
    't/lib/MessagePluginA.pm',
    't/lib/MessagePluginB.pm',
    't/test-dist-zilla-attributes.t',
    't/test-dist-zilla-build.t',
    't/test-dist-zilla-built-files.t',
    't/test-dist-zilla-exception.t',
    't/test-dist-zilla-files.t',
    't/test-dist-zilla-messages.t',
    't/test-dist-zilla-release.t',
    'xt/aspell-en.pws',
    'xt/perlcritic.ini'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
